import React from 'react'
const {useCallback, useEffect, useState, useRef} = React
export const useDebounceCallback = (callback, wait = 100, leading = false) => {
  const storedCallback = useRef(callback)
  storedCallback.current = callback
  const timeout = useRef(void 0)
  const deps = [wait, leading] // Cleans up pending timeouts when the deps change

  function _ref() {
    clearTimeout(timeout.current)
    timeout.current = void 0
  }

  useEffect(() => _ref, deps)

  function _ref2() {
    timeout.current = void 0
  }

  return useCallback(function () {
    // eslint-disable-next-line prefer-rest-params
    const args = arguments
    const current = timeout.current // Calls on leading edge

    if (current === void 0 && leading) {
      timeout.current = setTimeout(_ref2, wait) // eslint-disable-next-line prefer-spread

      return storedCallback.current.apply(null, args)
    } // Clear the timeout every call and start waiting again

    clearTimeout(current) // Waits for `wait` before invoking the callback

    timeout.current = setTimeout(() => {
      timeout.current = void 0 // eslint-disable-next-line prefer-spread

      storedCallback.current.apply(null, args)
    }, wait)
  }, deps)
}
export const useDebounce = (initialState, wait, leading) => {
  const _ref_0 = useState(initialState)

  const setState = _ref_0[1]
  const state = _ref_0[0]
  return [state, useDebounceCallback(setState, wait, leading)]
}
export default useDebounce
